# All known languages according to Wikipedia as of 2023
all_languages = [
    "afrikaans",
    "albanian",
    "amharic",
    "arabic",
    "armenian",
    "assamese",
    "azerbaijani",
    "basque",
    "belarusian",
    "bengali",
    "bhojpuri",
    "bosnian",
    "brazilian-portuguese",
    "bulgarian",
    "burmese",
    "cantonese",
    "catalan",
    "cebuano",
    "chechen",
    "chinese",
    "croatian",
    "czech",
    "danish",
    "dari",
    "dinka",
    "dutch",
    "english",
    "estonian",
    "finnish",
    "flemish",
    "french",
    "fula",
    "galician",
    "georgian",
    "german",
    "greek",
    "gujarati",
    "haitian",
    "hausa",
    "hebrew",
    "hiligaynon",
    "hindi",
    "hmong",
    "hungarian",
    "icelandic",
    "igbo",
    "ilokano",
    "indonesian",
    "irish",
    "italian",
    "japanese",
    "javanese",
    "kannada",
    "kazakh",
    "khmer",
    "kirundi",
    "korean",
    "kurdish",
    "kyrgyz",
    "lao",
    "latin",
    "latvian",
    "lithuanian",
    "luxembourgish",
    "macedonian",
    "madurese",
    "maithili",
    "malagasy",
    "malay",
    "malayalam",
    "maltese",
    "mandarin",
    "marathi",
    "mongolian",
    "nepalese",
    "norwegian",
    "odia",
    "oromo",
    "pangasinense",
    "pashto",
    "persian",
    "polish",
    "portuguese",
    "punjabi",
    "quechua",
    "romanian",
    "russian",
    "sardinian",
    "serbian",
    "serbo-croatian",
    "sindhi",
    "sinhala",
    "slovak",
    "slovene",
    "somali",
    "spanish",
    "sundanese",
    "swahili",
    "swedish",
    "tagalog",
    "tahitian",
    "taiwanese",
    "tamil",
    "tatar",
    "telugu",
    "thai",
    "tibetan",
    "turkish",
    "ukrainian",
    "urdu",
    "uyghur",
    "uzbek",
    "vietnamese",
    "welsh",
    "wolof",
    "wu",
    "xhosa",
    "yiddish",
    "yoruba",
    "zulu",
]
