import nltk

# Any NLTK packages that need to be specifically downloaded go here. NLTK will
# generally throw a readable error letting you know if something needs to be
# added to this list.
nltk.download("stopwords")
nltk.download("wordnet")
